library ELEKTOR_USB_UART;

// Interfac-DLL zur Einbindung der ELEKTOR-USB-UART
// (www.elektor.de ; Ausgabe Januar 2002)
// in ProfiLab Expert Software ab V3.0
// (www.abacom-online.de)

uses
  SysUtils,
  Windows,
  Classes,
  Controls,
  USB_In_Out in 'USB_IN_OUT.PAS',
  Dlg_ElektorConfig in 'Dlg_ElektorConfig.pas' {ConfigDlg};

{$R *.RES}

Const  Inputs = 36;  // number of inputs
       Outputs = 12; // number of outputs

       {DLL INPUTS}

       //Write Port0 Bit0..7
       W00   = 0;
       W01   = 1;
       W02   = 2;
       W03   = 3;
       W04   = 4;
       W05   = 5;
       W06   = 6;
       W07   = 7;

       //Write Port1 Bit0..3
       W10   = 8;
       W11   = 9;
       W12   = 10;
       W13   = 11;

       //PullUp Port0
       PU00   = 12;
       PU01   = 13;
       PU02   = 14;
       PU03   = 15;
       PU04   = 16;
       PU05   = 17;
       PU06   = 18;
       PU07   = 19;
       //PullUp Port1
       PU10   = 20;
       PU11   = 21;
       PU12   = 22;
       PU13   = 23;
       //I Sink Port0
       IS00   = 24;
       IS01   = 25;
       IS02   = 26;
       IS03   = 27;
       IS04   = 28;
       IS05   = 29;
       IS06   = 30;
       IS07   = 31;
       //I Sink Port1
       IS10   = 32;
       IS11   = 33;
       IS12   = 34;
       IS13   = 35;

       {DLL OUTPUTS}

       //Read Port0 Bit0..7
       R00   = 0;
       R01   = 1;
       R02   = 2;
       R03   = 3;
       R04   = 4;
       R05   = 5;
       R06   = 6;
       R07   = 7;
       //Read Port1 Bit0..3
       R10   = 8;
       R11   = 9;
       R12   = 10;
       R13   = 11;

       {USER}
       Port0_Old = 0;
       Port1_Old = 1;
       Port0_PullUp_Old = 2;
       Port1_PullUp_Old = 3;
       ISink00_Old = 10;
       ISink10_Old = 20;
       DevHandle = 30;
       DevNo = 40;


Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

function NumInputs: Byte;
begin
  result:=Inputs; //Define number of component input pins
end;

function NumOutputs: Byte;
begin
  result:=Outputs; //Define number of component output pins
end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   case Channel of
   W00: result:='W00';
   W01: result:='W01';
   W02: result:='W02';
   W03: result:='W03';
   W04: result:='W04';
   W05: result:='W05';
   W06: result:='W06';
   W07: result:='W07';

   W10: result:='W10';
   W11: result:='W11';
   W12: result:='W12';
   W13: result:='W13';

   PU00: result:='PU00';
   PU01: result:='PU01';
   PU02: result:='PU02';
   PU03: result:='PU03';
   PU04: result:='PU04';
   PU05: result:='PU05';
   PU06: result:='PU06';
   PU07: result:='PU07';

   PU10: result:='PU10';
   PU11: result:='PU11';
   PU12: result:='PU12';
   PU13: result:='PU13';

   IS00: result:='IS00';
   IS01: result:='IS01';
   IS02: result:='IS02';
   IS03: result:='IS03';
   IS04: result:='IS04';
   IS05: result:='IS05';
   IS06: result:='IS06';
   IS07: result:='IS07';

   IS10: result:='IS10';
   IS11: result:='IS11';
   IS12: result:='IS12';
   IS13: result:='IS13';
   end;
end;

Function OutputName(Channel: Byte): ShortString; // Return name for each component output pin
begin
   case Channel of
   R00: result:='R00';
   R01: result:='R01';
   R02: result:='R02';
   R03: result:='R03';
   R04: result:='R04';
   R05: result:='R05';
   R06: result:='R06';
   R07: result:='R07';

   R10: result:='R10';
   R11: result:='R11';
   R12: result:='R12';
   R13: result:='R13';
   end;
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
var i: integer;
    DevString: String;
begin
   // Make sure everything is set in first CALCULATE call
   PUser^[Port0_Old]:=-1;
   PUser^[Port1_Old]:=-1;
   PUser^[Port0_PullUp_Old]:=-1;
   PUser^[Port1_PullUp_Old]:=-1;
   For i:=0 to 7 do PUser^[ISink00_old+i]:=-1;
   For i:=0 to 3 do PUser^[ISink10_old+i]:=-1;
   //Open driver
   DevString:='\\.\usbuart_'+IntToStr(round(PUser^[DevNo]));
   DeviceHandle := CreateFile (PChar(DevString),Generic_write,File_Share_write,
                               nil,open_existing,0,TemplateHandle);

   PUser^[DevHandle]:=DeviceHandle; //Store DeviceHandle local for case of multiple DLL usage
end;

Procedure SimStop(PInput,POutput,PUser: PDLLParams); //called when ProfiLab RUN mode is terminated
begin
   //Close driver
   DeviceHandle:=round(PUser^[DevHandle]); //Restore DeviceHandle from local user variable before using it
   CloseHandle (DeviceHandle);
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var i: Integer;
    aByte: Byte;
    ISink: Extended;
begin

   DeviceHandle:=round(PUser^[DevHandle]); //Restore DeviceHandle from local user variable before using it

  // PullUp

   aByte:=0;
   For i:=0 to 7 do if PInput^[PU00+i]>2.5 then aByte:=(aByte or (1 shl i));
   if aByte<>PUser^[Port0_PullUp_old] then //only when changed...
   begin
      Port0PullUp(aByte);
      PUser^[Port0_PullUp_old]:=aByte; //Remeber status for next call
   end;

   aByte:=0;
   For i:=0 to 3 do if PInput^[PU10+i]>2.5 then aByte:=(aByte or (1 shl i));
   if aByte<>PUser^[Port1_PullUp_old] then //only when changed...
   begin
      Port1PullUp(aByte);
      PUser^[Port1_PullUp_old]:=aByte; //Remeber status for next call
   end;


   //I Sink
   For i:=0 to 7 do
   begin
      ISink:=PInput^[IS00+i];
      if ISink<0 then ISink:=0;
      if ISink>255 then ISink:=255;
      aByte:=round(ISink);
      if aByte<>PUser^[ISink00_old+i] then //only when changed...
      begin
         Port0ISink(i,aByte);
         PUser^[ISink00_old+i]:=aByte; //Remeber status for next call
      end;
   end;

   For i:=0 to 3 do
   begin
      ISink:=PInput^[IS10+i];
      if ISink<0 then ISink:=0;
      if ISink>255 then ISink:=255;
      aByte:=round(ISink);
      if aByte<>PUser^[ISink10_old+i] then //only when changed...
      begin
         Port1ISink(i,aByte);
         PUser^[ISink10_old+i]:=aByte; //Remeber status for next call
      end;
   end;

   //Write

   aByte:=0;
   For i:=0 to 7 do if PInput^[W00+i]>2.5 then aByte:=(aByte or (1 shl i));
   if aByte<>PUser^[Port0_old] then //only when changed...
   begin
      Port0Write(aByte);
      PUser^[Port0_old]:=aByte; //Remeber status for next call
   end;

   aByte:=0;
   For i:=0 to 3 do if PInput^[W10+i]>2.5 then aByte:=(aByte or (1 shl i));
   if aByte<>PUser^[Port1_old] then //only when changed...
   begin
      Port1Write(aByte);
      PUser^[Port1_old]:=aByte; //Remeber status for next call
   end;

   //Read

   aByte:=Port0Read;
   For i:=0 to 7 do if (aByte and (1 shl i))>0 then POutput^[R00+i]:=5 else POutput^[R00+i]:=0;

   aByte:=Port1Read;
   For i:=0 to 3 do if (aByte and (1 shl i))>0 then POutput^[R10+i]:=5 else POutput^[R10+i]:=0;
end;


Procedure Configure(PUser: PDLLParams);
var DLG: TConfigDlg;
begin
   DLG:=TConfigDlg.Create(nil);                       //Create user dialogue
   With DLG do
   begin
      SpinEdit1.Value:=round(PUser^[DevNo]);      // Set dialogue values
      if ShowModal=mrOK then                                 // Execute dialogue
      begin
         PUser^[DevNo]:=SpinEdit1.Value;     // Read values from dialogue
      end;
      free;                                                  // Free dialogue
   end;
end;

//export methods for ProfiLab
exports SimStart,
        SimStop,
        NumInputs,
        NumOutputs,
        Calculate,
        InputName,
        OutputName,
        Configure;
begin
end.

